-- Filename:     tb_dflipflops.vhd
-- Filetype:     VHDL Testbench Code
-- Date:         8 aug 2013
-- Update:       -
-- Description:  VHDL Testbench Code for dflipflops
-- Author:       J. op den Brouw
-- State:        Demo
-- Error:        -
-- Version:      1.0
-- Copyright:    (c)2013, De Haagse Hogeschool

-- This file contains a VHDL testbench code for the schematic entry
-- file dflipflops.bdf.

library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;

entity tb_dflipflops is
end entity tb_dflipflops;

architecture testbench of tb_dflipflops is
component dflipflops is
    port	(clk :    in std_logic;
          areset : in std_logic;
          x :      in std_logic;
          q0 :     out std_logic;
          q1 :     out std_logic
         );
end component dflipflops;
signal clk : std_logic;
signal areset : std_logic;
signal x : std_logic;
signal q0 : std_logic;
signal q1 : std_logic;

--constant freq_in : integer := 10000; -- 10 kHz
constant freq_in : integer := 50000000; -- 50 MHz
constant clock_period : time := (1.0/real(freq_in)) * (1 sec);
constant small_delay : time := 1 ns;
begin
  
  dut : dflipflops
  port map (clk => clk, areset => areset, x => x, q0 => q0, q1 => q1);

	-- The clock signal generation process
	clockgen: process is
	begin
		-- give time for clock to start
		clk <= '0';
		wait for 10 ns;
		-- forever: generate clock cycle for 20 ns and 50% d.c.
		loop
			clk <= '1';
			wait for clock_period/2;
			clk <= '0';
			wait for clock_period/2;
		end loop;
	end process;

	-- The data generation process
	datagen: process is
	-- All the valuations for the DUT as string. Note the construction: a
	-- constant may be unconstrained, but a variable may not. Thus the
	-- trick is to define an unconstrained constant array and assign it
	-- to a constrained variable using the range of the previously assigned
	-- constant.
	constant x_inputs_setup : std_logic_vector := "0110111111101001110";
	variable x_inputs : std_logic_vector(x_inputs_setup'range) := x_inputs_setup;
	begin
       -- Set input to known value
       x <= '0';

       -- Generation of the asynchronous reset signal.
		 -- Set low first and wait two clock cycles
       areset <= '0';
       wait for 2*clock_period;
	
		 -- Set high and wait two clock cycles
       areset <= '1';
       wait for 2*clock_period;

       -- Synchronize on the positive edge of the clock, and wait a bit.
       wait until clk = '1';
       wait for small_delay;

       -- Iterate over all array indices		  
		 for index in x_inputs'range loop  -- loop through all input bits
           x <= x_inputs(index);         -- assign to FSM input
           wait until clk = '1';
           wait for small_delay;
		 end loop;

       -- Wait forever
       wait;
	 end process datagen;
	 
end architecture testbench;